<?php
	if ( !class_exists('Twofactor') ) {
		class Twofactor
		{
			private $success_notice;
			private $success_message;
			private $error_notice;
			private $error_message;

			function __construct()
			{
				add_action('admin_menu',[$this,"add_admin_menu"]);
			}

			function add_admin_menu()
			{
				add_menu_page('vauth_twofactor','２段階認証','read',"vauth_twofactor",[$this,'menuMain'], 'dashicons-lock');
				add_menu_page( 'vauth_twofactor_input','vauth_twofactor', 'read', 'vauth_twofactor_input',   [$this,'inputTwofactor']);
				remove_menu_page('vauth_twofactor_input');
			}


			function menuMain()
			{
				$current_user = wp_get_current_user();
				global $wpdb;

				$vauth_result = $wpdb->get_results($wpdb->prepare("SELECT meta_key,meta_value  FROM {$wpdb->usermeta} WHERE user_id = %d AND meta_key LIKE 'vauth_%'", $current_user->ID),OBJECT_K);

				$vauth_code = $wpdb->get_results("SELECT meta_key,meta_value  FROM {$wpdb->usermeta} WHERE meta_key in('vauth_api_token','vauth_site_code')",OBJECT_K);

				$contact_list = [];
				$site = [];

				if(!empty($vauth_result)){
					$ValueAuth = new ValueAuth($current_user->ID);
					$contact_result = $ValueAuth->getContact();
					if(isset($contact_result['result']) && $contact_result['http_code'] == 200){
						$contact_list = $contact_result['result']->results->customer_contacts;
					}

					$site_result = $ValueAuth->getSite();
					if(isset($site_result['result']) && $site_result['http_code'] == 200){
						$site = $site_result['result']->results->site_setting;
					}
				}

				require_once 'elements/views/MainPage.php';
			}

			function setTwofactor($type)
			{
				if ($type === 'mail') {
					add_action('admin_menu',[$this,"setTwofactorMailAddress"]);
				} elseif ($type === 'sms') {
					add_action('admin_menu',[$this,"setTwofactorSmsAddress"]);
				} elseif ($type === 'setting') {
					add_action('admin_menu',[$this,"setTwofactorSetting"]);
				} elseif ($type === 'use') {
					add_action('admin_menu',[$this,"setTwofactorUse"]);
				}
			}

			function deleteTwofactor($type)
			{
				if ($type === 'mail') {
					add_action('admin_menu',[$this,"deleteTwofactorMailAddress"]);
				} elseif ($type === 'sms') {
					add_action('admin_menu',[$this,"deleteTwofactorSmsAddress"]);
				}
			}

			function deleteTwofactorSmsAddress(){
				$this->deleteTwofactorAddress('sms');
			}

			function deleteTwofactorMailAddress(){
				$this->deleteTwofactorAddress('mail');
			}

			function setTwofactorSmsAddress(){
				$this->setTwofactorAddress('sms');
			}

			function setTwofactorMailAddress(){
				$this->setTwofactorAddress('mail');
			}

			function setTwofactorUse(){
				global $wpdb;
				$vauth_use_name = "vauth_use";
				$user = wp_get_current_user();
				$use_flg = $_POST[$vauth_use_name] === 'on' ? 1 : 0;

				try {
					$umeta_id = $wpdb->get_var($wpdb->prepare("SELECT umeta_id FROM {$wpdb->usermeta} WHERE user_id = {$user->ID} AND meta_key = %s",$vauth_use_name));
					$wpdb->query("START TRANSACTION");
					if (empty($umeta_id)) {
						$wpdb->insert($wpdb->usermeta,
							['user_id' => $user->ID,'meta_key' => $vauth_use_name,'meta_value' => $use_flg],
							['%d','%s','%d']);
					} else {
						$wpdb->update($wpdb->usermeta,['meta_value' => $use_flg],
							['user_id' => $user->ID,'meta_key' => $vauth_use_name]);
					}
					$wpdb->query("COMMIT");
					add_action( 'admin_notices', [$this,"showTwofactorErrorNotices"] );
				}catch (Exception $e){
					$wpdb->query("ROLLBACK");
				}
			}

			function deleteTwofactorAddress($type)
			{
				global $wpdb;
				$user = wp_get_current_user();

				if (!isset($_POST['contact_id'])) {
				}

				$contact_id = $_POST['contact_id'];

				try {
					// change address
					$ValueAuth = new ValueAuth($user->ID);
					$result = $ValueAuth->deleteContact($type,$contact_id);
					if($result['http_code'] !== 200){
						throw new Exception($result['result']['message']);
					}
					$this->success_message = "削除しました。";
					add_action( 'admin_notices', [$this,"showTwofactorErrorNotices"]);
					$this->success_notice = true;
				}catch (Exception $e){
					$this->error_notice = true;
					$this->error_message = $e->getMessage();
					add_action( 'admin_notices', [$this,"showTwofactorErrorNotices"]);
				}
			}


			function setTwofactorAddress($type)
			{
				global $wpdb;
				$vauth_address_name = "vauth_{$type}_address";
				$user = wp_get_current_user();
				$vauth_address = $_POST[$vauth_address_name];

				try {
					// change address
					$ValueAuth = new ValueAuth($user->ID);

					if (!isset($_POST['contact_id'])) {
						$result = $ValueAuth->createContact($type,$vauth_address);
						if($result['http_code'] == 200){
							$this->success_message = "登録しました。";
						}else{
							if(!empty($result['result']->errors->message->address[0])){
								throw new Exception($result['result']->errors->message->address[0]);
							}elseif(!empty($result['result']->errors->message)){
								throw new Exception($result['result']->errors->message);
							}
						}
					} else {
						$wpdb->update($wpdb->usermeta,['meta_value' => $vauth_address],
							['user_id' => $user->ID,'meta_key' => $vauth_address_name]);
						$contact_id = (int)$_POST['contact_id'];
						$result = $ValueAuth->updateContact($type,$contact_id,$vauth_address);
						if($result['http_code'] !== 200){
							throw new Exception($result['result']->errors->message);
						}
						$this->success_message = "更新しました。";

					}
					add_action( 'admin_notices', [$this,"showTwofactorErrorNotices"]);
					$this->success_notice = true;
				}catch (Exception $e){
					$this->error_notice = true;
					$this->error_message = $e->getMessage();
					add_action( 'admin_notices', [$this,"showTwofactorErrorNotices"]);
				}
			}

			function setTwofactorSetting()
			{

				$user = wp_get_current_user();

				if(!in_array('administrator',$user->roles)){
					$this->error_notice = true;
					$this->error_message = "権限がありません。";
					add_action( 'admin_notices', [$this,"showTwofactorErrorNotices"] );
					return;
				}

				$vauth_site_code = $_POST['vauth_site_code'];
				$vauth_api_token = $_POST['vauth_api_token'];

				$ValueAuth = new ValueAuth($user->ID);
				$ValueAuth->setSiteCode($vauth_site_code);
				$ValueAuth->setApiToken($vauth_api_token);
				$result = $ValueAuth->getSite();
				$site = [];
				if (!empty($result['http_code']) AND $result['http_code'] == 200) {
					$this->updateSiteCode();
					$this->updateApiToken();
					$this->success_notice = true;
					$this->success_message = "サイト情報を登録しました。";
				}else{
					$this->error_notice = true;
					$this->error_message = $result['result']->errors->message;
				}
				add_action( 'admin_notices', [$this,"showTwofactorErrorNotices"] );
			}

			function updateSiteCode(){
				global $wpdb;
				$user_id = 0;
				$vauth_site_code = $_POST['vauth_site_code'];
				$umeta_id = $wpdb->get_var("SELECT umeta_id FROM {$wpdb->usermeta} WHERE user_id = {$user_id} AND meta_key = 'vauth_site_code'");

				if(empty($umeta_id)){
					$wpdb->insert( $wpdb->usermeta, ['user_id'=>$user_id,'meta_key'=>'vauth_site_code','meta_value'=>$vauth_site_code],['%d','%s','%s']);
				}else{
					$wpdb->update( $wpdb->usermeta, ['meta_value'=>$vauth_site_code],['user_id'=>$user_id,'meta_key'=>'vauth_site_code'],['%s'],['%d','%s']);
				}
			}

			function updateApiToken(){
				global $wpdb;
				$user_id = 0;
				$vauth_api_token = $_POST['vauth_api_token'];
				$umeta_id = $wpdb->get_var("SELECT umeta_id FROM {$wpdb->usermeta} WHERE user_id = {$user_id} AND meta_key = 'vauth_api_token'");

				if(empty($umeta_id)){
					$wpdb->insert( $wpdb->usermeta, ['user_id'=>$user_id,'meta_key'=>'vauth_api_token','meta_value'=>$vauth_api_token],['%d','%s','%s']);
				}else{
					$wpdb->update( $wpdb->usermeta, ['meta_value'=>$vauth_api_token],['user_id'=>$user_id,'meta_key'=>'vauth_api_token'],['%s'],['%d','%s']);
				}

			}

			/*
			 * 有効化された時にデータを作成する
			 *
			 */
			function activate()
			{
			}


			function checkTwofactorHook()
			{
				add_action( 'wp_login', [$this,"loginTwofactor"],10,2);
				add_action( 'admin_menu', [$this,"checkTwofactor"]);
			}

			function resendTwofactorHook()
			{
				add_action( 'admin_menu', [$this,"resendTwofactor"]);
			}

			function loginTwofactor( $user_login , $current_user )
			{
				global $wpdb;

				// no use valueauth
				$is_use_vauth = $wpdb->get_row("SELECT * FROM {$wpdb->usermeta} WHERE user_id = {$current_user->ID} AND meta_key LIKE 'vauth_use' AND meta_value = 1");
				if(empty($is_use_vauth)){
					return;
				}

				$ValueAuth = new ValueAuth($current_user->ID);
				$site_result = $ValueAuth->getSite();
				if(isset($site_result['result']) && $site_result['http_code'] != 200){
					return;
				}

				// send two_auth
				$this->sendTwofactor($current_user);
			}


			function resendTwofactor(){
				$current_user = wp_get_current_user();
				$result = $this->sendTwofactor($current_user,1);

				$admin_url = admin_url();
				$redirect_url = $admin_url."admin.php?page=vauth_twofactor_input";
				if(isset($result['http_code']) && $result['http_code'] == 200){
					wp_redirect($redirect_url.'&resend=success', 302 );
				}else{
					wp_redirect( $redirect_url.'&resend=error', 302 );
				}
				exit;
			}


			function checkTwofactor()
			{
				global $wpdb;
				$current_user = wp_get_current_user();
				$admin_url = admin_url();
				$check_url = (empty($_SERVER["HTTPS"]) ? "http://" : "https://") .$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'];

				// no use valueauth
				$is_use_vauth = $wpdb->get_row("SELECT * FROM {$wpdb->usermeta} WHERE user_id = {$current_user->ID} AND meta_key LIKE 'vauth_use' AND meta_value = 1");
				if(empty($is_use_vauth)){
					if(preg_match("/vauth_twofactor_input/",$check_url)){
						wp_redirect($admin_url,302);
						exit;
					}else{
						return;
					}
				}

				$is_lock_vauth = $wpdb->get_row("SELECT * FROM {$wpdb->usermeta} WHERE user_id = {$current_user->ID} AND meta_key LIKE 'vauth_lock' AND meta_value = 1");
				// lock valueauth
				if(empty($is_lock_vauth)){
					if(preg_match("/vauth_twofactor_input/",$check_url)){
						wp_redirect($admin_url,302);
						exit;
					}else{
						return;
					}
				}

				$redirect_url = $admin_url."admin.php?page=vauth_twofactor_input";
				$allow_urls = [];
				$allow_urls[] = $admin_url."admin.php?page=vauth_twofactor_input";
				$allow_urls[] = $admin_url."admin.php?page=vauth_twofactor_input&resend=success";
				$allow_urls[] = $admin_url."admin.php?page=vauth_twofactor_input&resend=error";
				$allow_urls[] = $admin_url."admin.php?page=vauth_twofactor_input&confirm=error";

				if(!in_array($check_url,$allow_urls)){
					wp_redirect($redirect_url,302);
					exit;
				}

				if(isset($_REQUEST['resend'])){
					if($_REQUEST['resend'] === "success") {
						$this->success_notice = true;
						$this->success_message = "認証コードを再送信しました。";
					}elseif($_REQUEST['resend'] === "error"){
						$this->error_notice = true;
						$this->error_message = "認証コードの再送信に失敗しました。";
					}
				}elseif($_REQUEST['confirm']) {
					if ($_REQUEST['confirm'] === "error") {
						$this->error_notice = true;
						$this->error_message = "認証コードが正しくありません。";
					}
				}
				add_action( 'admin_notices', [$this,"showTwofactorErrorNotices"] );

			}

			function sendTwofactor($current_user,$resend_flg=0){
				global $wpdb;
				$is_use_vauth = $wpdb->get_row("SELECT * FROM {$wpdb->usermeta} WHERE user_id = {$current_user->ID} AND meta_key LIKE 'vauth_use' AND meta_value = 1");
				$result = false;
				if(!empty($is_use_vauth)){
					$ValueAuth = new ValueAuth($current_user->ID);
					$result = $ValueAuth->sendTwofactor();
					if($result['http_code'] == 200 && !$resend_flg){
						$wpdb->insert( $wpdb->usermeta, ['user_id'=>$current_user->ID,'meta_key'=>'vauth_lock','meta_value'=>1],['%d','%s','%d']);
					}
				}
				return $result;
			}

			function confirmTwofactorHook()
			{
				add_action('after_setup_theme',[$this,"confirmTwofactor"],10,2);
			}


			function confirmTwofactor(){
				global $wpdb;
				$current_user = wp_get_current_user();
				$code = $_REQUEST["twofactor_code"];
				$ValueAuth = new ValueAuth($current_user->ID);
				$result = $ValueAuth->confirmTwofactorApi($code);
				if($result['http_code'] == 200){
					$result = $wpdb->delete( $wpdb->usermeta, ['user_id'=>$current_user->ID,'meta_key'=>'vauth_lock'],['%d','%s']);
					wp_redirect( '/wp-admin/', 302 );
					exit;
				}else{
					$admin_url = admin_url();
					$redirect_url = $admin_url."admin.php?page=vauth_twofactor_input";
					wp_redirect( $redirect_url.'&confirm=error', 302 );
					exit;
				}
			}

			function inputTwofactor()
			{
				require_once 'elements/views/TwofactorInput.php';
				exit;
			}


			function showTwofactorErrorNotices()
			{

				$html = "";
				if($this->success_notice){
					$message = $this->success_message ? $this->success_message : "成功しました。";
					$html .= "<div class='updated'>";
					$html .= "  <p>{$message}</p>";
					$html .= '</div>';
				}

				if($this->error_notice){
					$message = $this->error_message ? $this->error_message : "失敗しました。";
					$html .= "<div class='error'>";
					$html .= "  <p>{$message}</p>";
					$html .= '</div>';
				}

				echo $html;
			}

		}
	}
?>